;;;======================================================
;;;     Lunochod V 1.0.1
;;;     Common
;;;     For ECLIPS Version 6.3
;;;     2016.12.02 Victor Kazarinov
;;;======================================================
;;;    .    MAIN
;;;======================================================

(defglobal
     ?*gamefieldref* = 0              ; reference    .
     ?*statusline0YPos* = 21          ; Y      .
     ?*statusline1YPos* = 22          ; Y      .
     ?*statusline2YPos* = 23          ; Y      .
     ?*statusline3YPos* = 24          ; Y      .
     ?*statuslineForeColor* = 15      ;       .
     ?*statuslineBackColor* = 0       ;       .
     ?*backgroundSym* = 49            ;     .
     ?*backgroundSymForeColor* = 13   ;       .
     ?*backgroundSymBackColor* = 8    ;       .
     ?*gamefieldDimX* = 80            ;   .
     ?*gamefieldDimY* = 20            ;   .
     ?*barrierSym* = 48               ;  .
)

;;;======================================================
;   .

;           .
(deffunction field-fill (?dimX ?dimY ?cellsize ?array2dref)
 (loop-for-count (?y 0 (- ?dimY 1)) do
  (loop-for-count (?x 0 (- ?dimX 1)) do
   (tui-color-set ?*backgroundSymForeColor* ?*backgroundSymBackColor*)
   (tui-xy-goto ?x ?y)
   (sys-char-print (sys-2darray-read ?x ?y ?cellsize ?array2dref ?*gamefieldDimX*))
  )
 )
)

;;;======================================================

(deffunction tui_status-print (?srctext ?statusNum)
 (tui-color-set ?*statuslineForeColor* ?*statuslineBackColor*)
 (if (= ?statusNum 0)
  then
   (bind ?statuslineYPos ?*statusline0YPos*)
  else
   (if (= ?statusNum 1)
    then
     (bind ?statuslineYPos ?*statusline1YPos*)
    else
     (if (= ?statusNum 2)
      then
       (bind ?statuslineYPos ?*statusline2YPos*)
      else
       (bind ?statuslineYPos ?*statusline3YPos*)
      )
   )
 )
 (tui-xy-goto 0 ?statuslineYPos)
 (printout t "                                                                      ")
 (tui-xy-goto 0 ?statuslineYPos)
 (printout t ?srctext)
)

(deffunction agent-goto (?sym ?srcX ?srcY ?dstX ?dstY ?forecolor ?backcolor)
 (tui-color-set ?*backgroundSymForeColor* ?*backgroundSymBackColor*)
 (tui-xy-goto ?srcX ?srcY)
 (sys-char-print ?*backgroundSym*)                        ;  .
 (sys-2darray-write ?srcX ?srcY 1 ?*backgroundSym* ?*gamefieldref* ?*gamefieldDimX*)
 (tui-color-set ?forecolor ?backcolor)
 (tui-xy-goto ?dstX ?dstY)
 (sys-char-print ?sym)
 (sys-2darray-write ?dstX ?dstY 1 ?sym ?*gamefieldref* ?*gamefieldDimX*)
)

(deffunction sensor-barrier-detect (?x ?y ?direction ?array2dref ?DimX) ;  : absent -    ; present,  .
 (if (eq ?direction right)          ; .
  then
   (bind ?barrier (sys-2darray-read (+ ?x 1) ?y 1 ?array2dref ?DimX))
  else
   (if (eq ?direction up)           ; .
    then
    (bind ?barrier (sys-2darray-read ?x (- ?y 1) 1 ?array2dref ?DimX))
    else
     (if (eq ?direction down)       ; .
      then
       (bind ?barrier (sys-2darray-read ?x (+ ?y 1) 1 ?array2dref ?DimX))
      else                          ; .
       (bind ?barrier (sys-2darray-read (- ?x 1) ?y 1 ?array2dref ?DimX))
      )
   )
 )
 (if (eq ?barrier ?*backgroundSym*)
  then
;   (tui_status-print "sensor-barrier-detect: barrier absent" 2)
   (bind ?result absent)
  else
;   (tui_status-print "sensor-barrier-detect: barrier present" 2)
   (bind ?result present)
 )
)

;;;======================================================
;  

(deftemplate agent-position                 ;     
 (slot agent-name)                          ;  .
 (slot posx)
 (slot posy)
)

(deftemplate agent-position-next            ;     
 (slot agent-name)                          ;  .
 (slot posx (type INTEGER))
 (slot posy (type INTEGER))
)

(deftemplate agent-direction                ;      
 (slot agent-name)                          ;  .
 (slot direction)
)

(deftemplate agent-direction-next           ;      
 (slot agent-name)                          ;  .
 (slot direction)
)

(deftemplate agent-image                    ;      
 (slot agent-name)                          ;  .
 (slot sym)                                 ;  ,   
 (slot color (type INTEGER))                ;  .
)

(deftemplate agent-image-next               ;      
 (slot agent-name)                          ;  .
 (slot sym)                                 ;  ,   
 (slot color (type INTEGER))                ;  .
)


(deftemplate  agent-action                  ;   ,    EXECUTOR
 (slot agent-name)                          ;  .
 (slot mission-name)                        ;  .
 (slot action-name)                         ;  .
 (slot param)                               ;  .
)

(deftemplate agent-answer                   ;    ,    EXECUTOR
 (slot agent-name)                          ;  .
 (slot mission-name)                        ;  .
 (slot action-name)                         ;  .
 (slot status)                              ;  .
)

(deftemplate agent-sensor                   ;    
 (slot agent-name)                          ;  .
 (slot sensor-name)                         ;  .
 (slot param)                               ;  .
)


;;;======================================================
;  

(deftemplate mission-command                ;   ,    SOLVER.
 (slot agent-name)                          ;  ,     .
 (slot mission-name)                        ;  .
 (slot command-name)                        ;  ,   .
 (slot param (type INTEGER) (default 0))
)

(deftemplate  mission-answer                ;   ,    EXECUTOR
 (slot agent-name)                          ;  ,    .
 (slot mission-name)                        ;  .
 (slot command-name)                        ;  ,   .
 (slot status)                              ;  .
)

(deftemplate mission-phase                  ;     .
 (slot agent-name)                          ;  ,    .
 (slot mission-name)                        ;  .
 (slot phase)                               ;   .
)